#pragma once

#include <J3ML/LinearAlgebra/Matrix4x4.h>
#include <J3ML/LinearAlgebra/Vector3.h>
#include <J3ML/LinearAlgebra/Quaternion.h>
#include <cassert>
#include <vector>
#include "assimp/anim.h"

namespace LearnOpenGL
{
    using namespace J3ML::LinearAlgebra;

    struct KeyPosition
    {
        Vector3 Position;
        float Timestamp;
    };

    struct KeyRotation
    {
        Quaternion Orientation;
        float Timestamp;
    };

    struct KeyScale
    {
        Vector3 Scale;
        float Timestamp;
    };

    class Bone
    {
    public:
        Bone(const std::string& name, int ID, const aiNodeAnim* channel);
        void Update(float animationTime);
        Matrix4x4 GetLocalTransform() const;
        std::string GetBoneName() const;
        int GetBoneID() const;
        int GetPositionIndex(float animationTime);
        int GetRotationIndex(float animationTime);
        int GetScaleIndex(float animationTime);
        // Private?
        static float GetScaleFactor(float lastTimestamp, float nextTimestamp, float animationTime);
        Matrix4x4 InterpolatePosition(float animationTime);
        Matrix4x4 InterpolateRotation(float animationTime);
        Matrix4x4 InterpolateScaling(float animationTime);


        std::vector<KeyPosition> Positions;
        std::vector<KeyRotation> Rotations;
        std::vector<KeyScale> Scales;
        int NumPositions;
        int NumRotations;
        int NumScalings;

        Matrix4x4 LocalTransform;
        std::string Name;
        int ID;
    };
}